using System;
using System.Collections.Generic;
using System.Text;
using TestSearch;
using System.Windows.Forms;
using System.Diagnostics;

namespace GuiSearch
{
    class QuickListController
    {
        private int _topIndex = -1;
        private int _bottomIndex = -1;
        private IIndexer<string> _indexer;
        private IList<string> _allLines;
        private IList<string> _currentResults;
        private QuickList _listControl;
        List<ListViewItem> _itemCache = new List<ListViewItem>();
        private string _filter;
        
        #region Properties
        public IList<string> CurrentResults
        {
            get
            {
                return _currentResults;
            }
        }
        public string FilterText
        {
            get
            {
                return _filter;
            }
            set
            {
                _filter = value;
            }
        }
        public IIndexer<string> Indexer
        {
            get
            {
                return _indexer;
            }
            set
            {
                _indexer = value;
            }
        }
        public IList<string> AllLines
        {
            get
            {
                return _allLines;
            }
            set
            {
                _allLines = value;
            }
        }
        #endregion

        public QuickListController(QuickList list)
        {
            _listControl = list;
            _listControl.VirtualMode = true;
            _listControl.CacheVirtualItems += new System.Windows.Forms.CacheVirtualItemsEventHandler(list_CacheVirtualItems);
            _listControl.RetrieveVirtualItem += new System.Windows.Forms.RetrieveVirtualItemEventHandler(list_RetrieveVirtualItem);
        }

        
        void list_CacheVirtualItems(object sender, System.Windows.Forms.CacheVirtualItemsEventArgs e)
        {
            _topIndex = e.StartIndex;
            _bottomIndex = e.EndIndex;

            int sizeNeeded = _bottomIndex - _topIndex + 2;//extra 2
            int toCreate = sizeNeeded - _itemCache.Count;
            if (toCreate > 0)
            {
                if (_itemCache.Capacity < sizeNeeded)
                {
                    _itemCache.Capacity += toCreate;
                }
                for (int i = 0; i < toCreate; i++)
                {
                    _itemCache.Add(new ListViewItem());
                }
            }
        }

        void list_RetrieveVirtualItem(object sender, System.Windows.Forms.RetrieveVirtualItemEventArgs e)
        {
            if (e.ItemIndex < _topIndex)
            {
                //should never get here
                Debug.Assert(false);
                _topIndex = e.ItemIndex;
            }
            
            e.Item = GetListViewItem(e.ItemIndex, _topIndex);
        }

        private ListViewItem GetListViewItem(int itemIndex, int topIndex)
        {
            if (itemIndex < 0 || topIndex < 0)
            {
                return null;
            }
            int cacheIndex = itemIndex - topIndex;
            
            Debug.Assert(cacheIndex < _itemCache.Count);
            ListViewItem item = _itemCache[cacheIndex];
            item.Text = _currentResults[itemIndex];
            //could set tag if you have more complex objects
            //item.Tag = _currentResults[itemIndex];
            return item;
        }

        public void RefreshList()
        {
            if (string.IsNullOrEmpty(_filter))
            {
                _currentResults = _allLines;
            }
            else
            {
                _currentResults = _indexer.Lookup(_filter);
            }
            _listControl.VirtualListSize = _currentResults.Count;
        }

    }
}
