using System;
using System.Collections.Generic;
using System.Text;

namespace TestSearch
{
    /// <summary>
    /// NaiveIndexer is merely an example of using IIndexer--it should not be used
    /// </summary>
    /// <typeparam name="T">Type of value to index</typeparam>
    public class NaiveIndexer<T> : IIndexer<T>
    {
        private struct ItemStruct
        {
            public string _key;
            public T _value;
            //public ushort _sortOrder;
        };

        List<ItemStruct> _items = new List<ItemStruct>();

        #region IIndexer<T> Members

        public void AddItem(string key, T value, UInt32 sortOrder)
        {
            string realKey = RemoveUnneededCharacters(key);
            ItemStruct itemStruct = new ItemStruct();
            itemStruct._key = realKey;
            itemStruct._value = value;
            //itemStruct._sortOrder = sortOrder;
            //could insert into place of sortOrder, after a grow, if desired
            _items.Add(itemStruct);
        }

        public IList<T> Lookup(string subKey)
        {
            List<T> results = new List<T>();
            foreach (ItemStruct itemStruct in _items)
            {
                if (itemStruct._key.IndexOf(subKey, StringComparison.InvariantCultureIgnoreCase)>=0)
                {
                    results.Add(itemStruct._value);
                }
            }
            //Sort on results[i]._sortOrder, if desired
            return results;
        }

        public void FinishIndex()
        {
            //don't need to do anything
        }

        #endregion

        /// <summary>
        /// Removes all but alphanumeric characters from the input string
        /// </summary>
        /// <param name="original">The original.</param>
        /// <returns>A string with all unneeded characters removed</returns>
        private string RemoveUnneededCharacters(string original)
        {
            char[] array = new char[original.Length];
            int destIndex = 0;
            for (int i = 0; i < original.Length; i++)
            {
                char c = original[i];
                if (char.IsLetterOrDigit(c))
                {
                    array[destIndex] = c;
                    destIndex++;
                }
            }
            return new string(array, 0, destIndex);
        }
    }
}
